@@ -105,7 +105,7 @@ static NEOERR *alt_parse (CSPARSE *parse, int cmd, char *arg);
 static NEOERR *alt_eval (CSPARSE *parse, CSTREE *node, CSTREE **next);
 
 static NEOERR *render_node (CSPARSE *parse, CSTREE *node);
-static NEOERR *cs_init_internal (CSPARSE **parse, HDF *hdf, BOOL init_funcs);
+static NEOERR *cs_init_internal (CSPARSE **parse, HDF *hdf, CSPARSE *parent);
 static int rearrange_for_call(CSARG **args);
 
 typedef struct _cmds
@@ -192,6 +192,7 @@ static NEOERR *alloc_node (CSTREE **node)
   return STATUS_OK;
 }
 
+/* TODO: make these deallocations linear and not recursive */
 static void dealloc_arg (CSARG **arg)
 {
   CSARG *p;
@@ -276,11 +277,11 @@ static int find_open_delim (CSPARSE *parse, char *buf, int x, int len)
   return -1;
 }
 
-NEOERR *cs_parse_file (CSPARSE *parse, char *path)
+NEOERR *cs_parse_file (CSPARSE *parse, const char *path)
 {
   NEOERR *err;
   char *ibuf;
-  char *save_context;
+  const char *save_context;
   int save_infile;
   char fpath[_POSIX_PATH_MAX];
 
@@ -579,6 +580,7 @@ static HDF *var_lookup_obj (CSPARSE *parse, char *name)
 {
   CS_LOCAL_MAP *map;
   char *c;
+  HDF *ret_hdf;
 
   map = lookup_map (parse, name, &c);
   if (map && map->type == CS_TYPE_VAR)
@@ -592,7 +594,13 @@ static HDF *var_lookup_obj (CSPARSE *parse, char *name)
       return hdf_get_obj (map->h, c+1);
     }
   }
-  return hdf_get_obj (parse->hdf, name);
+  /* smarti:  Added support for global hdf under local hdf */
+  /* return hdf_get_obj (parse->hdf, name); */
+  ret_hdf = hdf_get_obj (parse->hdf, name);
+  if (ret_hdf == NULL && parse->global_hdf != NULL) {
+    ret_hdf = hdf_get_obj (parse->global_hdf, name);
+  }
+  return ret_hdf;
 }
 
 /* Ugh, I have to write the same walking code because I can't grab the
@@ -657,6 +665,7 @@ static char *var_lookup (CSPARSE *parse, char *name)
 {
   CS_LOCAL_MAP *map;
   char *c;
+  char* retval;
 
   map = lookup_map (parse, name, &c);
   if (map) 
@@ -691,7 +700,13 @@ static char *var_lookup (CSPARSE *parse, char *name)
       return map->s;
     }
   }
-  return hdf_get_value (parse->hdf, name, NULL);
+  /* smarti:  Added support for global hdf under local hdf */
+  /* return hdf_get_value (parse->hdf, name, NULL); */
+  retval = hdf_get_value (parse->hdf, name, NULL);
+  if (retval == NULL && parse->global_hdf != NULL) {
+    retval = hdf_get_value (parse->global_hdf, name, NULL);
+  }
+  return retval;
 }
 
 long int var_int_lookup (CSPARSE *parse, char *name)
@@ -1447,7 +1462,7 @@ static NEOERR *evar_parse (CSPARSE *parse, int cmd, char *arg)
   NEOERR *err;
   CSTREE *node;
   char *a, *s;
-  char *save_context;
+  const char *save_context;
   int save_infile;
   char tmp[256];
 
@@ -2099,7 +2114,7 @@ static NEOERR *lvar_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
 
       /* Ok, we need our own copy of the string to pass to
        * cs_parse_string... */
-      if (val.alloc) {
+      if (val.alloc && (val.op_type & CS_TYPE_STRING)) {
 	val.alloc = 0;
       }
       else 
@@ -2112,15 +2127,13 @@ static NEOERR *lvar_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
       }
 
       do {
-	err = cs_init_internal(&cs, parse->hdf, FALSE);
+	err = cs_init_internal(&cs, parse->hdf, parse);
 	if (err) break;
-	cs->functions = parse->functions;
 	err = cs_parse_string(cs, s, strlen(s));
 	if (err) break;
 	err = cs_render(cs, parse->output_ctx, parse->output_cb);
 	if (err) break;
       } while (0);
-      cs->functions = NULL;
       cs_destroy(&cs);
     }
   }
@@ -2154,9 +2167,8 @@ static NEOERR *linclude_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
     {
       CSPARSE *cs = NULL;
       do {
-	err = cs_init_internal(&cs, parse->hdf, FALSE);
+	err = cs_init_internal(&cs, parse->hdf, parse);
 	if (err) break;
-	cs->functions = parse->functions;
 	err = cs_parse_file(cs, s);
 	if (!(node->flags & CSF_REQUIRED))
 	{
@@ -2166,7 +2178,6 @@ static NEOERR *linclude_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
 	err = cs_render(cs, parse->output_ctx, parse->output_cb);
 	if (err) break;
       } while (0);
-      cs->functions = NULL;
       cs_destroy(&cs);
     }
   }
@@ -2381,6 +2392,8 @@ static NEOERR *each_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
       each_map.type = CS_TYPE_VAR;
       each_map.name = node->arg1.s;
       each_map.next = parse->locals;
+      each_map.first = 1;
+      each_map.last = 0;
       parse->locals = &each_map;
 
       do
@@ -2388,12 +2401,16 @@ static NEOERR *each_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
 	child = hdf_obj_child (var);
 	while (child != NULL)
 	{
+          /* We don't explicitly set each_map.last here since checking
+           * requires a function call, so we move the check to _builtin_last
+           * so it only makes the call if last() is being used */
 	  each_map.h = child;
 	  err = render_node (parse, node->case_0);
           if (each_map.map_alloc) {
             free(each_map.s);
             each_map.s = NULL;
           }
+          if (each_map.first) each_map.first = 0;
 	  if (err != STATUS_OK) break;
 	  child = hdf_obj_next (child);
 	}
@@ -2748,7 +2765,7 @@ static NEOERR *call_parse (CSPARSE *parse, int cmd, char *arg)
   {
     err = nerr_raise (NERR_PARSE, 
 	"%s Incorrect number of arguments, expected %d, got %d in call to macro %s: %s",
-	find_context(parse, -1, tmp, sizeof(tmp)), macro->n_args, x, 
+	find_context(parse, -1, tmp, sizeof(tmp)), macro->n_args, nargs, 
 	macro->name, arg);
   }
   if (err)
@@ -2774,11 +2791,18 @@ static NEOERR *call_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   int x;
 
   macro = node->arg1.macro;
-  call_map = (CS_LOCAL_MAP *) calloc (macro->n_args, sizeof(CS_LOCAL_MAP));
-  if (call_map == NULL)
-    return nerr_raise (NERR_NOMEM, 
-	"Unable to allocate memory for call_map in call_eval of %s", 
-	macro->name);
+  if (macro->n_args)
+  {
+    call_map = (CS_LOCAL_MAP *) calloc (macro->n_args, sizeof(CS_LOCAL_MAP));
+    if (call_map == NULL)
+      return nerr_raise (NERR_NOMEM, 
+                "Unable to allocate memory for call_map in call_eval of %s", 
+                         macro->name);
+  }
+  else
+  {
+    call_map = NULL;
+  }
 
   darg = macro->args;
   carg = node->vargs;
@@ -2853,7 +2877,7 @@ static NEOERR *call_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
   {
     if (call_map[x].map_alloc) free(call_map[x].s);
   }
-  free (call_map);
+  if (call_map) free (call_map);
 
   *next = node->next;
   return nerr_pass(err);
@@ -3116,17 +3140,20 @@ static NEOERR *loop_eval (CSPARSE *parse, CSTREE *node, CSTREE **next)
     each_map.type = CS_TYPE_NUM;
     each_map.name = node->arg1.s;
     each_map.next = parse->locals;
+    each_map.first = 1;
     parse->locals = &each_map;
 
     var = start;
     for (x = 0, var = start; x < iter; x++, var += step)
     {
+      if (x == iter - 1) each_map.last = 1;
       each_map.n = var;
       err = render_node (parse, node->case_0);
       if (each_map.map_alloc) {
         free(each_map.s);
         each_map.s = NULL;
       }
+      if (each_map.first) each_map.first = 0;
       if (err != STATUS_OK) break;
     } 
 
@@ -3172,7 +3199,8 @@ NEOERR *cs_render (CSPARSE *parse, void *ctx, CSOUTFUNC cb)
 
 /* **** Functions ******************************************** */
 
-static NEOERR *_register_function(CSPARSE *parse, char *funcname, int n_args, CSFUNCTION function)
+static NEOERR *_register_function(CSPARSE *parse, const char *funcname,
+                                  int n_args, CSFUNCTION function)
 {
   CS_FUNCTION *csf;
 
@@ -3207,90 +3235,6 @@ static NEOERR *_register_function(CSPARSE *parse, char *funcname, int n_args, CS
   return STATUS_OK;
 }
 
-static NEOERR * _builtin_subcount(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
-{
-  HDF *obj;
-  int count = 0;
-
-  result->op_type = CS_TYPE_NUM;
-  result->n = 0;
-
-  if (args->op_type & CS_TYPE_VAR)
-  {
-    obj = var_lookup_obj (parse, args->s);
-    if (obj != NULL)
-    {
-      obj = hdf_obj_child(obj);
-      while (obj != NULL)
-      {
-	count++;
-	obj = hdf_obj_next(obj);
-      }
-    }
-    result->n = count;
-  }
-  else
-  {
-    // everything else has zero children
-    result->n = 0;
-  }
-
-  return STATUS_OK;
-}
-
-static NEOERR * _builtin_str_length(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
-{
-  HDF *obj;
-
-  result->op_type = CS_TYPE_NUM;
-  result->n = 0;
-
-  if (args->op_type & CS_TYPE_VAR)
-  {
-    obj = var_lookup_obj (parse, args->s);
-    if (obj) {
-      char *s = hdf_obj_value(obj);
-      if (s) {
-	result->n = strlen(s);
-      }
-    }
-  }
-  else if (args->op_type & CS_TYPE_STRING)
-  {
-    result->n = strlen(args->s);
-  }
-  return STATUS_OK;
-}
-
-
-static NEOERR * _builtin_name(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
-{
-  HDF *obj;
-
-  result->op_type = CS_TYPE_STRING;
-  result->s = "";
-
-  if (args->op_type & CS_TYPE_VAR)
-  {
-    obj = var_lookup_obj (parse, args->s);
-    if (obj != NULL)
-    {
-      result->s = hdf_obj_name(obj);
-    }
-    else 
-    {
-      result->s = "";
-    }
-  }
-  else if (args->op_type & CS_TYPE_STRING)
-  {
-    result->s = args->s;
-    result->alloc = args->alloc;
-    args->alloc = 0;
-  }
-  return STATUS_OK;
-}
-
 /* This is similar to python's PyArg_ParseTuple, :
  *   s - string (allocated)
  *   i - int
@@ -3354,14 +3298,196 @@ static NEOERR * cs_arg_parse(CSPARSE *parse, CSARG *args, char *fmt, ...)
   return nerr_pass(err);
 }
 
+static NEOERR * _builtin_subcount(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
+{
+  HDF *obj;
+  int count = 0;
+
+  /* default for non-vars is 0 children */
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  if (args->op_type & CS_TYPE_VAR)
+  {
+    obj = var_lookup_obj (parse, args->s);
+    if (obj != NULL)
+    {
+      obj = hdf_obj_child(obj);
+      while (obj != NULL)
+      {
+	count++;
+	obj = hdf_obj_next(obj);
+      }
+    }
+    result->n = count;
+  }
+
+  return STATUS_OK;
+}
+
+static NEOERR * _builtin_str_length(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
+{
+  HDF *obj;
+
+  /* non var/string objects have 0 length */
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  if (args->op_type & CS_TYPE_VAR)
+  {
+    obj = var_lookup_obj (parse, args->s);
+    if (obj) {
+      char *s = hdf_obj_value(obj);
+      if (s) {
+	result->n = strlen(s);
+      }
+    }
+  }
+  else if (args->op_type & CS_TYPE_STRING)
+  {
+    result->n = strlen(args->s);
+  }
+  return STATUS_OK;
+}
+
+
+static NEOERR * _builtin_name(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
+{
+  HDF *obj;
+
+  result->op_type = CS_TYPE_STRING;
+  result->s = "";
+
+  if (args->op_type & CS_TYPE_VAR)
+  {
+    obj = var_lookup_obj (parse, args->s);
+    if (obj != NULL)
+    {
+      result->s = hdf_obj_name(obj);
+    }
+    else 
+    {
+      result->s = "";
+    }
+  }
+  else if (args->op_type & CS_TYPE_STRING)
+  {
+    result->s = args->s;
+    result->alloc = args->alloc;
+    args->alloc = 0;
+  }
+  return STATUS_OK;
+}
+
+/* Check to see if a local variable is the first in an each/loop sequence */
+static NEOERR * _builtin_first(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args,
+                               CSARG *result)
+{
+  CS_LOCAL_MAP *map;
+  char *c;
+
+  /* default is "not first" */
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  /* Only applies to possible local vars */
+  if ((args->op_type & CS_TYPE_VAR) && !strchr(args->s, '.'))
+  {
+    map = lookup_map (parse, args->s, &c);
+    if (map && map->first)
+      result->n = 1;
+  }
+  return STATUS_OK;
+}
+
+/* Check to see if a local variable is the last in an each/loop sequence */
+/* TODO: consider making this work on regular HDF vars */
+static NEOERR * _builtin_last(CSPARSE *parse, CS_FUNCTION *csf, CSARG *args,
+                               CSARG *result)
+{
+  CS_LOCAL_MAP *map;
+  char *c;
+
+  /* default is "not last" */
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  /* Only applies to possible local vars */
+  if ((args->op_type & CS_TYPE_VAR) && !strchr(args->s, '.'))
+  {
+    map = lookup_map (parse, args->s, &c);
+    if (map) {
+      if (map->last) {
+        result->n = 1;
+      } else if (map->type == CS_TYPE_VAR) {
+        if (hdf_obj_next(map->h) == NULL) {
+          result->n = 1;
+        }
+      }
+    }
+  }
+  return STATUS_OK;
+}
+
+/* returns the absolute value (ie, positive) of a number */
+static NEOERR * _builtin_abs (CSPARSE *parse, CS_FUNCTION *csf, CSARG *args,
+                              CSARG *result)
+{
+  int n1 = 0;
+
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  n1 = arg_eval_num(parse, args);
+  result->n = abs(n1);
+
+  return STATUS_OK;
+}
+
+/* returns the larger or two integers */
+static NEOERR * _builtin_max (CSPARSE *parse, CS_FUNCTION *csf, CSARG *args,
+                              CSARG *result)
+{
+  NEOERR *err;
+  long int n1 = 0;
+  long int n2 = 0;
+
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  err = cs_arg_parse(parse, args, "ii", &n1, &n2);
+  if (err) return nerr_pass(err);
+  result->n = (n1 > n2) ? n1 : n2;
+
+  return STATUS_OK;
+}
+
+/* returns the smaller or two integers */
+static NEOERR * _builtin_min (CSPARSE *parse, CS_FUNCTION *csf, CSARG *args,
+                              CSARG *result)
+{
+  NEOERR *err;
+  long int n1 = 0;
+  long int n2 = 0;
+
+  result->op_type = CS_TYPE_NUM;
+  result->n = 0;
+
+  err = cs_arg_parse(parse, args, "ii", &n1, &n2);
+  if (err) return nerr_pass(err);
+  result->n = (n1 < n2) ? n1 : n2;
+
+  return STATUS_OK;
+}
+
 static NEOERR * _builtin_str_slice (CSPARSE *parse, CS_FUNCTION *csf, CSARG *args, CSARG *result)
 {
   NEOERR *err;
   char *s = NULL;
   char *slice;
-  int b = 0;
-  int e = 0;
-  int len;
+  long int b = 0;
+  long int e = 0;
+  size_t len;
 
   result->op_type = CS_TYPE_STRING;
   result->s = "";
@@ -3475,10 +3601,10 @@ NEOERR *cs_register_strfunc(CSPARSE *parse, char *funcname, CSSTRFUNC str_func)
 
 /* **** CS Initialize/Destroy ************************************ */
 NEOERR *cs_init (CSPARSE **parse, HDF *hdf) {
-  return nerr_pass(cs_init_internal(parse, hdf, TRUE));
+  return nerr_pass(cs_init_internal(parse, hdf, NULL));
 }
 
-static NEOERR *cs_init_internal (CSPARSE **parse, HDF *hdf, BOOL init_funcs)
+static NEOERR *cs_init_internal (CSPARSE **parse, HDF *hdf, CSPARSE *parent)
 {
   NEOERR *err = STATUS_OK;
   CSPARSE *my_parse;
@@ -3528,53 +3654,61 @@ static NEOERR *cs_init_internal (CSPARSE **parse, HDF *hdf, BOOL init_funcs)
     cs_destroy(&my_parse);
     return nerr_pass(err);
   }
-  if (init_funcs)
-  {
-    err = _register_function(my_parse, "len", 1, _builtin_subcount);
-    if (err)
-    {
-      cs_destroy(&my_parse);
-      return nerr_pass(err);
-    }
-    err = _register_function(my_parse, "subcount", 1, _builtin_subcount);
-    if (err)
-    {
-      cs_destroy(&my_parse);
-      return nerr_pass(err);
-    }
-    err = _register_function(my_parse, "name", 1, _builtin_name);
-    if (err)
-    {
-      cs_destroy(&my_parse);
-      return nerr_pass(err);
-    }
-    err = _register_function(my_parse, "string.slice", 3, _builtin_str_slice);
-    if (err)
-    {
-      cs_destroy(&my_parse);
-      return nerr_pass(err);
-    }
-    err = _register_function(my_parse, "string.length", 1, _builtin_str_length);
-    if (err)
-    {
-      cs_destroy(&my_parse);
-      return nerr_pass(err);
-    }
-
-
-  }
-#ifdef ENABLE_GETTEXT
-  err = _register_function(my_parse, "_", 1, _builtin_gettext);
-  if (err)
-  {
-    cs_destroy(&my_parse);
-    return nerr_pass(err);
-  }
-#endif
   my_parse->tag = hdf_get_value(hdf, "Config.TagStart", "cs");
   my_parse->taglen = strlen(my_parse->tag);
   my_parse->hdf = hdf;
 
+  if (parent == NULL)
+  {
+    static struct _builtin_functions {
+      const char *name;
+      int nargs;
+      CSFUNCTION function;
+    } Builtins[] = {
+      { "len", 1, _builtin_subcount },
+      { "subcount", 1, _builtin_subcount },
+      { "name", 1, _builtin_name },
+      { "first", 1, _builtin_first },
+      { "last", 1, _builtin_last },
+      { "abs", 1, _builtin_abs },
+      { "max", 2, _builtin_max },
+      { "min", 2, _builtin_min },
+      { "string.slice", 3, _builtin_str_slice },
+      { "string.length", 1, _builtin_str_length },
+#ifdef ENABLE_GETTEXT
+      { "_", 1, _builtin_gettext },
+#endif
+      { NULL, 0, NULL },
+    };
+    int x = 0;
+    while (Builtins[x].name != NULL) {
+      err = _register_function(my_parse, Builtins[x].name, Builtins[x].nargs,
+                               Builtins[x].function);
+      if (err)
+      {
+        cs_destroy(&my_parse);
+        return nerr_pass(err);
+      }
+      x++;
+    }
+    /* Set global_hdf to be null */
+    my_parse->global_hdf = NULL;
+    my_parse->parent = NULL;
+  }
+  else
+  {
+    /* TODO: macros and functions should actually not be duplicated, they
+     * should just be modified in lookup to walk the CS struct hierarchy we're 
+     * creating here */
+    /* BUG: We currently can't copy the macros because they reference the parse
+     * tree, so if this sub-parse tree adds a macro, the macro reference will
+     * persist, but the parse tree it points to will be gone when the sub-parse
+     * is gone. */
+    my_parse->functions = parent->functions;
+    my_parse->global_hdf = parent->global_hdf;
+    my_parse->parent = parent;
+  }
+
   *parse = my_parse;
   return STATUS_OK;
 }
@@ -3591,7 +3725,9 @@ void cs_destroy (CSPARSE **parse)
 
   dealloc_macro(&my_parse->macros);
   dealloc_node(&(my_parse->tree));
-  dealloc_function(&(my_parse->functions));
+  if (my_parse->parent == NULL) {
+    dealloc_function(&(my_parse->functions));
+  }
 
   free(my_parse);
   *parse = NULL;
@@ -3806,4 +3942,3 @@ NEOERR *cs_dump_c (CSPARSE *parse, char *path)
   return nerr_pass (err);
 }
 #endif
-
