@@ -13,6 +13,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <time.h>
 #include <limits.h>
 #include "util/neo_misc.h"
@@ -31,7 +32,7 @@
  * than this */
 static char TzBuf[_POSIX_PATH_MAX + 4];
 
-static int time_set_tz (char *timezone)
+static int time_set_tz (const char *timezone)
 {
   snprintf (TzBuf, sizeof(TzBuf), "TZ=%s", timezone);
   putenv(TzBuf);
@@ -39,21 +40,37 @@ static int time_set_tz (char *timezone)
   return 0;
 }
 
-void neo_time_expand (const time_t tt, char *timezone, struct tm *ttm)
+void neo_time_expand (const time_t tt, const char *timezone, struct tm *ttm)
 {
-  time_set_tz (timezone);
+  const char *cur_tz = getenv("TZ");
+  int change_back = 0;
+  if (cur_tz == NULL || strcmp(timezone, cur_tz)) {
+    time_set_tz (timezone);
+    change_back = 1;
+  }
   localtime_r (&tt, ttm);
+  if (cur_tz && change_back) {
+    time_set_tz(cur_tz);
+  }
 }
 
-time_t neo_time_compact (struct tm *ttm, char *timezone)
+time_t neo_time_compact (struct tm *ttm, const char *timezone)
 {
   time_t r;
   int save_isdst = ttm->tm_isdst;
 
-  time_set_tz (timezone);
+  const char *cur_tz = getenv("TZ");
+  int change_back = 0;
+  if (cur_tz == NULL || strcmp(timezone, cur_tz)) {
+    time_set_tz (timezone);
+    change_back = 1;
+  }
   ttm->tm_isdst = -1;
   r = mktime(ttm);
   ttm->tm_isdst = save_isdst;
+  if (cur_tz && change_back) {
+    time_set_tz(cur_tz);
+  }
   return r;
 }
 
